// Loop through and create add functions
// Accepted parameters:
//      ObjectName          - Required name of object
//      RequestParse        - Optional function to manipulate request params
//      ResponseCallback    - Optional function to be called when completed
var addFunctions = {
    "addEvent": { "ObjectName": "Event" },
    "addPost": { "ObjectName": "Post" },
    "addAccount": { "ObjectName": "Account" },
    "addConfiguration": { "ObjectName": "Configuration" },
    "addScreen": { "ObjectName": "Screen" }
};

for (var functionName in addFunctions) {
    Parse.Cloud.define(functionName, function(functionName) {
        return function(request, response) {
            var addObject = new Parse.Object(addFunctions[functionName].ObjectName);

            if (typeof addFunctions[functionName].RequestParse === 'function') {
                request.params = addFunctions[functionName].RequestParse(request.params);
            }

            for (var key in request.params) {
                addObject.set(key, request.params[key]);
            }

            addObject.save(null, {
                success: function(addedObject) {
                    if (typeof addFunctions[functionName].ResponseCallback === 'function') {
                        request.params = addFunctions[functionName].ResponseCallback(addedObject);
                    }

                    response.success(JSON.stringify({ 'status': 'success', 'message': addedObject }));
                }, error: function(er) {
                    if (typeof addFunctions[functionName].ResponseCallback === 'function') {
                        request.params = addFunctions[functionName].ResponseCallback(false);
                    }

                    response.error("Error saving event. Error #" + er.code + ": " + er.message);
                }
            });
        }
    }(functionName));
}

// Loop through and create update functions
// Accepted parameters:
//      ObjectName          - Required name of object
//      RequestParse        - Optional function to manipulate request params
//      ResponseCallback    - Optional function to be called when completed
var updateFunctions = {
    "updateEvent": { "ObjectName": "Event" },
    "updatePost": { "ObjectName": "Post" },
    "updateAccount": { "ObjectName": "Account" },
    "updateConfiguration": { "ObjectName": "Configuration" },
    "updateScreen": { "ObjectName": "Screen" }
};

for (var functionName in updateFunctions) {
    Parse.Cloud.define(functionName, function(functionName) {
        return function(request, response) {
            var updateQuery = new Parse.Query(updateFunctions[functionName].ObjectName);

            updateQuery.get(request.params.objectId, {
                success: function(objectToUpdate) {
                    if (typeof updateFunctions[functionName].RequestParse === 'function') {
                        request.params = updateFunctions[functionName].RequestParse(request.params);
                    }

                    for (var key in request.params) {
                        objectToUpdate.set(key, request.params[key]);
                    }

                    objectToUpdate.save(null, {
                        success: function(updatedObject) {
                            if (typeof updateFunctions[functionName].ResponseCallback === 'function') {
                                request.params = updateFunctions[functionName].ResponseCallback(updatedObject);
                            }

                            response.success(JSON.stringify({ 'status': 'success', 'message': updatedObject }));
                        },
                        error: function(er) {
                            if (typeof updateFunctions[functionName].ResponseCallback === 'function') {
                                request.params = updateFunctions[functionName].ResponseCallback(false);
                            }

                            response.error("There was an error saving the object. Error #" + er.code + ":" + er.message);
                        }
                    });
                },
                error: function(er)
                {
                    response.error("There was an error retrieving the object. Error #" + er.code + ":" + er.message);
                }
            });
        }
    }(functionName));
}

// Loop through and create get functions
// Accepted parameters:
//      ObjectName          - Required name of object
//      ResponseCallback    - Optional function to be called when completed
var getFunctions = {
    "getEvents": { "ObjectName": "Event" },
    "getUsers": { "ObjectName": "_User" },
    "getPosts": { "ObjectName": "Post" },
    "getDonations": { "ObjectName": "Donation" },
    "getScreens": { "ObjectName": "Screen" }
};

for (var functionName in getFunctions) {
    Parse.Cloud.define(functionName, function(functionName) {
        return function(request, response) {
            var query = new Parse.Query(getFunctions[functionName].ObjectName);
            query.find({
                success: function(result) {
                    if (typeof getFunctions[functionName].ResponseCallback === 'function') {
                        getFunctions[functionName].ResponseCallback(result);
                    }

                    response.success(result);
                },
                error: function(er) {
                    if (typeof getFunctions[functionName].ResponseCallback === 'function') {
                        getFunctions[functionName].ResponseCallback(false);
                    }

                    response.error("Error retrieving results. Error #" + er.code + ": " + er.message);
                }
            });
        }
    }(functionName));
}
