<?php
namespace voteRockit;

class voteRockit {

  public $appId;
  public $restApiKey;
  protected $pointer;
  protected $queryUrl = 'https://api.parse.com/1/functions/';
  protected $pushUrl = 'https://api.parse.com/1/push/';
  protected $fileUploadUrl = 'https://api.parse.com/1/files/';
  protected $addUrl;
  protected $getUrl;
  protected $updateUrl;
  protected $validQueryTypes = array(
    'getEvents',
    'getUsers',
    'getPosts',
    'getDonations',
    'getScreens',
    'getAnalyticsPerDay',
    'getGearItems',
    'getOrderedGearItems',
    'getNeighbors'
  );
  protected $validateErrors;
  protected $dataTable; // Contains the key names and validation rules
  public $voterockitErrors = ''; //contains errors


  // Provides generic cURL connect options. Returns cURL handler.
  protected function connect($options) {
    // If data isn't being passed, create false data since service requires it
    $data = isset($options['CURLOPT_POSTFIELDS']) ? $options['CURLOPT_POSTFIELDS'] : json_encode(array('1' => '1'));

    // Build headers
    $headers = array();
    $headers[] = "X-Parse-Application-Id: $this->appId";
    $headers[] = "X-Parse-REST-API-Key: $this->restApiKey";

    // content length
    if (gettype($data) === 'string') {
      $headers[] = 'Content-Length: ' . strlen($data);
    }

    // Merge default headers with headers passed in options
    if (is_array($options['CURLOPT_HTTPHEADER'])) {
      $headers = array_merge($headers, $options['CURLOPT_HTTPHEADER']);
    }
    else {
      //if header not set then by default set JSON type
      $headers[] = 'Content-Type: application/json';
    }

    $handle = curl_init();
    curl_setopt($handle, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($handle, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($handle, CURLOPT_URL, $options['CURLOPT_URL']);
    curl_setopt($handle, CURLOPT_CUSTOMREQUEST, 'POST');
    curl_setopt($handle, CURLOPT_POSTFIELDS, $data);

    //bypass SSL
    curl_setopt($handle, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($handle, CURLOPT_SSL_VERIFYPEER, 0);

    $response = curl_exec($handle);
    $status = curl_getinfo($handle, CURLINFO_HTTP_CODE);

    // If there was a problem, pass to error handler
    if ($status != 200 && $status != 201) {
      $this->errorHandler($status, "Call to URL {$options['CURLOPT_URL']} failed with status $status, response $response, curl_error " . curl_error($handle) . ", curl_errno " . curl_errno($handle));
    }

    curl_close($handle);

    return array(
      'status'  => $status,
      'payload' => json_decode($response, TRUE)
    );
  }

  // Manages parse.com file uploads. Files are uploaded separately from other
  // calls so this just returns an array with where parse put the file
  public function uploadFile( /* string */
    $file) {

    if (!file_exists($file)) {
      $this->errorHandler('403', 'Could not read file. Is it missing or are permissions set improperly?');

      return FALSE;
    }

    // Get mime type to pass in header
    $file_info = new \finfo(FILEINFO_MIME);
    $mime_type = $file_info->buffer(file_get_contents($file));
    $file_mime = explode(';', $mime_type);
    $file_mime = $file_mime[0];

    $ds = '/';
    if (PHP_OS == 'WINNT') {
      $ds = '\\';
    }

    $fileArray = explode($ds, $file);
    $fileName = $this->sanitize(end($fileArray));

    $response = $this->connect(array(
      'CURLOPT_HTTPHEADER' => array('Content-Type: ' . $file_mime),
      'CURLOPT_URL'        => $this->fileUploadUrl . $fileName,
      'CURLOPT_POSTFIELDS' => file_get_contents($file),
    ));

    return array_merge(array('__type' => 'File'), $response['payload']);
  }

  /**
  * Function: sanitize
  * Returns a sanitized string, typically for URLs.
  *
  * Parameters:
  *     $string - The string to sanitize.
  *     $force_lowercase - Force the string to lowercase?
  *     $anal - If set to *true*, will remove all non-alphanumeric characters.
  */
  function sanitize($string, $force_lowercase = true, $anal = false) {
      $strip = array("~", "`", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "=", "+", "[", "{", "]",
                     "}", "\\", "|", ";", ":", "\"", "'", "&#8216;", "&#8217;", "&#8220;", "&#8221;", "&#8211;", "&#8212;",
                     "—", "–", ",", "<", ">", "/", "?");
      $clean = trim(str_replace($strip, "", strip_tags($string)));
      $clean = preg_replace('/\s+/', "-", $clean);
      $clean = ($anal) ? preg_replace("/[^a-zA-Z0-9]/", "", $clean) : $clean ;
      $clean = ($force_lowercase) ?
          (function_exists('mb_strtolower')) ?
              mb_strtolower($clean, 'UTF-8') :
              strtolower($clean) :
          $clean;
      return urlencode($clean);
  }

  // Generic error handler so that all errors look 'the same'
  protected function errorHandler( /* string */
    $status = 'error', /* string */
    $message) {

    $this->voterockitErrors = json_encode(
      array(
        'status'  => $status,
        'message' => $message
      )
    );

  }

  // Validates a single input
  protected function validateInput($key, $value, $validationType, $validationValue) {
    switch ($validationType) {
      case 'required':
        if (is_array($value)) {

          if ($value['__type'] == 'Date') {
            $value = $value['iso'];
          }
          elseif ($value['__type'] == 'File') {
            $value = $value['url'];
          }
          elseif ($value['__type'] == 'Pointer') {
            $value = $value['objectId'];
          }
        }

        if ($validationValue && !trim($value)) {
          $this->validateErrors[] = array(
            'keyName'    => $key,
            'validation' => 'required',
            'message'    => $key . ' is a required field'
          );

          return FALSE;
        }

        break;

      case 'regex':
        if ($validationValue && preg_match($validationValue, $value)) {
          $this->validateErrors[] = array(
            'keyName'    => $key,
            'validation' => 'regex',
            'message'    => $key . ' has an invalid value'
          );

          return FALSE;
        }

        break;

      case 'isDate':
        $parsedDate = date_parse($value);
        if (!checkdate($parsedDate['month'], $parsedDate['day'], $parsedDate['year'])) {
          $this->validateErrors[] = array(
            'keyName'    => $key,
            'validation' => 'isDate',
            'message'    => $key . ' is not a valid date'
          );
        }

        break;

      case 'isAlphaNumeric':
        if (!ctype_alnum($value)) {
          $this->validateErrors[] = array(
            'keyName'    => $key,
            'validation' => 'isDate',
            'message'    => $key . ' can only be letters and/or numbers'
          );
        }
        break;

      case 'isNumeric':
        if (!is_numeric($value)) {
          $this->validateErrors[] = array(
            'keyName'    => $key,
            'validation' => 'isDate',
            'message'    => $key . ' can only be a number'
          );
        }

        break;
    }

    return TRUE;
  }

  // Validates all inputs before being passed on
  protected function validate( /* array */
    $dataToValidate) {
    // Find fields that are completely missing
    foreach ($this->dataTable as $key => $value) {
      if (isset($this->dataTable[$key]['validation']) && $this->dataTable[$key]['validation']['required'] && !isset($dataToValidate[$key])) {
        $this->validationErrors[] = array(
          'keyName'    => $key,
          'validation' => 'required',
          'message'    => $key . ' is a required field'
        );
      }
    }

    // Loop through entire data table and run validation rules
    foreach ($dataToValidate as $key => $value) {
      if ($this->dataTable[$key]['validation']) {
        foreach ($this->dataTable[$key]['validation'] as $validationType => $validationValue) {
          $this->validateInput($key, $value, $validationType, $validationValue);
        }
      }
    }

    // Returns true if there were no errors
    return count($this->validationErrors) <= 0;
  }

  // Adds a row to an object
  public function add( /* array */
    $data) {
    if (!$this->validate($data)) {
      $this->errorHandler('400', json_encode($this->validationErrors));

      return FALSE;
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->addUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }

  // Gets a single row from an object table
  public function get( /* string */
    $pointer) {

  }

  // Updates a row in an object table
  public function update( /* array */
    $data) {
    if (!isset($data[$this->pointer])) {
      $this->errorHandler('400', 'Pointer must be provided for an update');

      return FALSE;
    }

    if (!$this->validate($data)) {
      $this->errorHandler('400', json_encode($this->validationErrors));

      return FALSE;
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->updateUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }

  // Gets a full list of objects from a table
  public function query( /* string */
    $queryType) {
    if (!in_array($queryType, $this->validQueryTypes)) {
      $this->errorHandler('fail', 'Not a valid query type');

      return FALSE;
    }

    $response = $this->connect(array('CURLOPT_URL' => $this->queryUrl . $queryType));

    return $response['payload'];
  }

}

class posts extends voteRockit {

  function __construct() {
    $this->addUrl = 'https://api.parse.com/1/functions/addPost';
    $this->updateUrl = 'https://api.parse.com/1/functions/updatePost';
    $this->pointer = 'objectId';
    $this->dataTable = array(
      'author'           => array('validation' => array('required' => TRUE)),
      'body'             => array('validation' => array('required' => TRUE)),
      'publishDate'      => array('validation' => array('required' => TRUE)),
      'shareEnabled'     => array('validation' => array('required' => TRUE)),
      'shortDescription' => array('validation' => array('required' => TRUE)),
      'title'            => array('validation' => array('required' => TRUE)),
      'status'           => array('validation' => array('required' => TRUE)),
      'type'             => array('validation' => array('required' => TRUE)),
      'hero'             => array('validation' => array('required' => FALSE)),
    );
  }

  // Override voteRockit add method so that we can upload the hero image
  public function add( /* array */
    $data) {
    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    if (isset($data['hero'])) {
      $data['hero'] = $this->uploadFile($data['hero']);
    }

    if (isset($data['video'])) {
      $data['video'] = $this->uploadFile($data['video']);
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->addUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }

  // Override voteRockit update method so that we can re-upload the hero image
  public function update( /* array */
    $data) {

    if (!isset($data[$this->pointer])) {
      $this->errorHandler('fail', 'Pointer must be provided for an update');

      return FALSE;
    }

    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    if (isset($data['hero'])) {
      $data['hero'] = $this->uploadFile($data['hero']);
    }

    if (isset($data['video'])) {
      $data['video'] = $this->uploadFile($data['video']);
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->updateUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }
}

class campaign extends voteRockit {

  function __construct() {
    $this->addUrl = 'https://api.parse.com/1/functions/addCampaign';
    $this->updateUrl = 'https://api.parse.com/1/functions/updateCampaign';
    $this->pointer = 'objectId';
    $this->dataTable = array(
      'address'         => array('validation' => array('required' => TRUE)),
      'city'            => array('validation' => array('required' => TRUE)),
      'state'           => array('validation' => array('required' => TRUE)),
      'zip'             => array('validation' => array('required' => TRUE)),
      'phone'           => array('validation' => array('required' => TRUE)),
      'campaignAddress' => array('validation' => array('required' => TRUE)),
      'campaignCity'    => array('validation' => array('required' => TRUE)),
      'campaignState'   => array('validation' => array('required' => TRUE)),
      'campaignZip'     => array('validation' => array('required' => TRUE)),
      'campaignPhone'   => array('validation' => array('required' => TRUE)),
      'contactName'     => array('validation' => array('required' => TRUE)),
      'contactEmail'    => array('validation' => array('required' => TRUE))
    );
  }
}

class events extends voteRockit {

  function __construct() {
    $this->addUrl = 'https://api.parse.com/1/functions/addEvent';
    $this->updateUrl = 'https://api.parse.com/1/functions/updateEvent';
    $this->pointer = 'objectId';
    $this->dataTable = array(
      'endDate'          => array('validation' => array('required' => TRUE)),
      'eventDescription' => array('validation' => array('required' => TRUE)),
      'eventImage'       => array('validation' => array('required' => FALSE)),
      'shortDescription' => array('validation' => array('required' => TRUE)),
      'startDate'        => array('validation' => array('required' => TRUE)),
      'status'           => array('validation' => array('required' => TRUE)),
      'title'            => array('validation' => array('required' => TRUE))
    );
  }

  // Override voteRockit add method so that we can upload the event image
  public function add( /* array */
    $data) {
    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    if (isset($data['eventImage'])) {
      $data['eventImage'] = $this->uploadFile($data['eventImage']);
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->addUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }

  // Override voteRockit update method so that we can re-upload the event image
  public function update( /* array */
    $data) {
    if (!isset($data[$this->pointer])) {
      $this->errorHandler('fail', 'Pointer must be provided for an update');

      return FALSE;
    }

    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    if (isset($data['eventImage'])) {
      $data['eventImage'] = $this->uploadFile($data['eventImage']);
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->updateUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }
}

class configuration extends voteRockit {

  function __construct() {
    $this->addUrl = 'https://api.parse.com/1/functions/addConfiguration';
    $this->updateUrl = 'https://api.parse.com/1/functions/updateConfiguration';
    $this->pointer = 'objectId';

    $this->dataTable = array(
      'campaign'                    => array('validation' => array('required' => TRUE)),
      'electionDate'                => array('validation' => array('required' => TRUE)),
      'getToKnowPhoto'              => array('validation' => array('required' => FALSE)),
      'lastConfigurationChangeDate' => array('validation' => array('required' => TRUE)),
      'localizationFile'            => array('validation' => array('required' => FALSE)),
      'logo'                        => array('validation' => array('required' => FALSE)),
      'stripeKey'                   => array('validation' => array('required' => TRUE)),
      'themeAccentColor'            => array('validation' => array('required' => TRUE)),
      'themeHighlightColor'         => array('validation' => array('required' => TRUE)),
      'themeMainColor'              => array('validation' => array('required' => TRUE)),
      'volunteerPhoto'              => array('validation' => array('required' => FALSE))
    );
  }

  // Override voteRockit add method so that we can upload the event image
  public function add( /* array */
    $data) {

    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    if (isset($data['getToKnowPhoto'])) {
      $data['getToKnowPhoto'] = $this->uploadFile($data['getToKnowPhoto']);
    }

    if (isset($data['localizationFile'])) {
      $data['localizationFile'] = $this->uploadFile($data['localizationFile']);
    }

    if (isset($data['logo'])) {
      $data['logo'] = $this->uploadFile($data['logo']);
    }

    if (isset($data['volunteerPhoto'])) {
      $data['volunteerPhoto'] = $this->uploadFile($data['volunteerPhoto']);
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->addUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }

  // Override voteRockit update method so that we can re-upload the event image
  public function update( /* array */
    $data) {
    if (!isset($data[$this->pointer])) {
      $this->errorHandler('fail', 'Pointer must be provided for an update');

      return FALSE;
    }

    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    if (isset($data['getToKnowPhoto'])) {
      $data['getToKnowPhoto'] = $this->uploadFile($data['getToKnowPhoto']);
    }

    if (isset($data['localizationFile'])) {
      $data['localizationFile'] = $this->uploadFile($data['localizationFile']);
    }

    if (isset($data['logo'])) {
      $data['logo'] = $this->uploadFile($data['logo']);
    }

    if (isset($data['volunteerPhoto'])) {
      $data['volunteerPhoto'] = $this->uploadFile($data['volunteerPhoto']);
    }


    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->updateUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }
}

class gears extends voteRockit {

  function __construct() {
    $this->addUrl = 'https://api.parse.com/1/functions/addGearItem';
    $this->updateUrl = 'https://api.parse.com/1/functions/updateGearItem';
    $this->pointer = 'objectId';
    $this->dataTable = array(
      'price' => array('validation' => array('required' => TRUE)),
      'photo' => array('validation' => array('required' => FALSE)),
      'title' => array('validation' => array('required' => TRUE))
    );
  }

  // Override voteRockit add method so that we can upload the gear image
  public function add( /* array */
    $data) {
    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    if (isset($data['photo'])) {
      $data['photo'] = $this->uploadFile($data['photo']);
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->addUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }

  // Override voteRockit update method so that we can re-upload the gear image
  public function update( /* array */
    $data) {
    if (!isset($data[$this->pointer])) {
      $this->errorHandler('fail', 'Pointer must be provided for an update');

      return FALSE;
    }

    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    if (isset($data['photo'])) {
      $data['photo'] = $this->uploadFile($data['photo']);
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->updateUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }
}
class neighbor extends voteRockit {

  function __construct() {
    $this->addUrl = 'https://api.parse.com/1/functions/addNeighbor';
    $this->dataTable = array(
      'address'    => array('validation' => array('required' => TRUE)),
      'city'       => array('validation' => array('required' => TRUE)),
      'email'      => array('validation' => array('required' => TRUE)),
      'geo'        => array('validation' => array('required' => FALSE)),
      'name'       => array('validation' => array('required' => TRUE)),
      'state'      => array('validation' => array('required' => TRUE)),
      'voteStatus' => array('validation' => array('required' => TRUE)),
      'zip'        => array('validation' => array('required' => TRUE)),
    );
  }

  public function add( /* array */
    $data) {
    if (!$this->validate($data)) {
      $this->errorHandler('fail', json_encode($this->validationErrors));

      return FALSE;
    }

    $response = $this->connect(array(
      'CURLOPT_URL'        => $this->addUrl,
      'CURLOPT_POSTFIELDS' => json_encode($data)
    ));

    return $response['payload'];
  }

}

class push extends voteRockit {

    public function send( /* array */
        $data) {

        $query = array();

        $query['where'] = array(
            'channels' => array(
                '$all' => $data['channels']
            ),
            'deviceType' => array(
                '$in' => $data['deviceType']
            )
        );

        $query['expiration_time'] = date('Y-m-d\Th:i:s\Z', $data['expiration_time']);

        $query['data'] = array(
            'alert' => $data['alert']
        );

        $response = $this->connect(array(
            'CURLOPT_URL'        => $this->pushUrl,
            'CURLOPT_POSTFIELDS' => json_encode($query)
        ));

        return $response['payload'];
    }

}
?>
