<?php
/**
 * Setting form for VoteRockIt
 * @see voterockit_setting_form_submit
 *
 */
function voterockit_setting_form($form, &$form_state) {

  drupal_add_css(VOTEROCKIT_PATH . '/includes/bootstrap/css/bootstrap.min.css', array(
    'group' => CSS_DEFAULT,
    'type'  => 'file'
  ));
  drupal_add_js(BASE_URL . VOTEROCKIT_PATH . '/includes/js/voterockit.js', array(
    'group' => JS_DEFAULT,
    'type'  => 'file'
  ));

  $js = "
  (function ($) {
    voterockit_cache_clear = function(id) {

    var r = confirm('" . t('Delete data?') . "');

    if (r == true) {
       voterockit_load({
          dataType: 'json',
          url: '" . url('admin/voterockit/cache/clear/', array('absolute' => TRUE)) . "'+id,
          type:'GET',
          loaderElement: null,
          success: function(response) {
             window.location = response.url;
          }
        });
    }
  }
  })(jQuery);
  ";

  drupal_add_js($js, 'inline');

  $form['voterockit'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('VoteRockIt Global Settings'),
    '#weight'      => 0,
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE,
  );

  $form['voterockit']['api_key'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Application Key"),
    '#required'      => TRUE,
    '#description'   => t("This is the main identifier that uniquely specifies your application. This is paired with a key to provide your clients access to your application's data."),
    '#size'          => 60,
    '#maxlength'     => 255,
    '#default_value' => variable_get('voterockit_application_key', '')
  );

  $form['voterockit']['rest_key'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Rest API Key"),
    '#required'      => TRUE,
    '#description'   => t("This key is used when making requests to the REST API. It also adheres to object level permissions."),
    '#size'          => 60,
    '#maxlength'     => 255,
    '#default_value' => variable_get('voterockit_rest_key', '')
  );

  $form['voterockit']['submit'] = array(
    '#type'  => 'submit',
    '#value' => t('Save'),
  );

  $form['voterockit_cache'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('VoteRockIt Cache'),
    '#weight'      => 1,
    '#collapsible' => TRUE,
    '#collapsed'   => TRUE,
  );

  $form['voterockit_cache']['voterockit_cache_clear'] = array(
    '#markup' => '<div style="float:left;margin-bottom:10px"><div style="float:left"><button class="btn btn-danger" type="button" onclick="voterockit_cache_clear(\'config\')" >Clear Configuration</button></div><div style="float:left;margin-left:10px"><button class="btn btn-danger" type="button" onclick="voterockit_cache_clear(\'content\')">Clear Content</button></div></div>',
  );

  return $form;

}

/**
 * submit callback
 * @see voterockit_setting_form
 *
 */
function voterockit_setting_form_submit($form, &$form_state) {

  $post = $form_state['values'];

  $api_key = check_plain(trim($post['api_key']));
  $rest_key = check_plain(trim($post['rest_key']));

  variable_set('voterockit_application_key', $api_key);
  variable_set('voterockit_rest_key', $rest_key);

  drupal_set_message(t('VoteRockIt global settings have been saved.'));
}

/**
 * clear table data
 */
function voterockit_cache_clear($id) {

  $id = trim($id);


  if (!user_access('administer VoteRockIt global configuration')) {
    drupal_set_message(t('Access denied.', 'error'));
  }
  else {

    $result = FALSE;

    if ($id == 'config') {
      $result = db_query("TRUNCATE TABLE {voterockit_config}");
    }
    elseif ($id == 'content') {
      $result = db_query("TRUNCATE TABLE {voterockit_content}");
    }
    else {
      drupal_set_message(t('Invalid index value.', 'error'));
    }

    if ($result == TRUE) {
      drupal_set_message(t('Data has been removed.'));
    }
    else {
      drupal_set_message(t('Error removing data', 'error'));
    }
  }

  echo json_encode(array('url' => url('admin/voterockit/setting', array('absolute' => TRUE))));
  exit;

}