<?php
/**
 * callback to show analytic page
 */
function voterockit_analytic_page() {

  voterockit_add_style_script();
  voterockit_prepare_highcharts();
  voterockit_prepare_grid();

  drupal_add_library('system', 'effects.blind');

  drupal_add_js(BASE_URL . VOTEROCKIT_ANALYTIC_PATH . '/includes/js/voterockit_analytic.js', array(
    'group' => JS_DEFAULT,
    'type'  => 'file'
  ));

  return theme('voterockit_analytic_interface');
}

/**
 * generate analytic interface
 *
 * @param array $data
 */
function theme_voterockit_analytic_interface() {
  return '
    <div class="row-fluid show_grid well" style="margin:0;">
      <div class="span12">
        <div class="row-fluid show_grid voterockit_analytic_top">
          <div class="span6 well well-small" id="voterockit_analytic_name"><h1></h1></div>
          <div class="span3" id="voterockit_analytic_loader"></div>
          <div class="span3">' . voterockit_analytic_create_dd() . '</div>
        </div>
        <div class="row-fluid show_grid voterockit_analytic_middle">
          <div class="span12" id="voterockit_analytic_actions">
            <button id="voterockit_analytic_chart_button" class="btn btn-info" type="button"><i class="icon-adjust icon-white"></i> ' . t('Chart') . '</button>
            <button id="voterockit_analytic_grid_button" class="btn btn-info" type="button"><i class="icon-th icon-white"></i> ' . t('Grid') . '</button>
          </div>
        </div>
        <div class="row-fluid show_grid voterockit_analytic_display" style="min-height:500px">
          <div id="voterockit_analytic_message"></div>
          <div id="voterockit_analytic_chart_display"></div>
          <div id="voterockit_analytic_grid_display"></div>
        </div>
      </div>
    </div>
  ';
}

/**
 * generate analytic dropdown
 */
function voterockit_analytic_create_dd() {

  return '
    <div class="btn-group" id="voterockit_analytic_dd">
      <a class="btn btn-primary" href="#"><i class="icon-tasks icon-white"></i> ' . t('Other Charts') . '</a>
      <a class="btn btn-primary dropdown-toggle" data-toggle="dropdown" href="#"><span class="caret"></span></a>
      <ul class="dropdown-menu" style="margin:2px 0px; padding:0px;">
        <li><a title="Donations and App Opens Per Day" class="voterockit_analytic_delegate first" href="' . url('voterockit/analytic/d/vapd') . '"><i class="icon-th-list"></i> ' . t('Per Day') . '</a></li>
        <li><a title="Voter Statistics" class="voterockit_analytic_delegate" href="' . url('voterockit/analytic/d/vav') . '"><i class="icon-user"></i> ' . t('Voters') . '</a></li>
        <li><a title="Merchandise Orders" class="voterockit_analytic_delegate" href="' . url('voterockit/analytic/d/vag') . '"><i class="icon-shopping-cart"></i> ' . t('Merchandise Orders') . '</a></li>
      </ul>
    </div>
  ';
}

/**
 * callback to delegate page request to appropriate analytic, requested by user
 *
 * @param string $name analytic identifier
 */
function voterockit_analytic_delegate($identifier) {

  if (is_ajax_request() === FALSE) {
    drupal_goto('admin/voterockit/analytic');
  }

  switch (trim($identifier)) {

    case 'vapd':
      $identifier = 'per_day';
      break;
    case 'vav':
      $identifier = 'voter';
      break;
    case 'vag':
      $identifier = 'gears';
      break;

  }
  //check permission
  if (!user_access('voterockit_analytic_' . $identifier)) {
    echo json_encode(array(
      'status'  => 'fail',
      'message' => t('Access denied.'),
    ));
    exit;
  }

  $variables = array();

  $return = call_user_func('voterockit_analytic_' . $identifier, $variables);

  echo json_encode($return);
  exit;

}

/**
 * per day analytic
 *
 * @param array $data
 */
function voterockit_analytic_per_day($data) {

  $chart = array();

  $voteorckit = create_voterockit_object(VOTEROCKIT_BASE);
  $results = $voteorckit->query('getAnalyticsPerDay');
  $results = $results['result'];

  if (empty($results)) {
    return array(
      'status'  => 'fail',
      'message' => t('Parse data not available or unreachable.'),
    );
  }

  $chart = array(
    'chart'   => array(
      'type'         => 'line',
      'marginRight'  => 130,
      'marginBottom' => 20,
      'height'       => 400,
    ),
    'title'   => array(
      'text' => t('Donations and App Opens Per Day'),
      'x'    => -20,
    ),
    'legend'  => array(
      'layout'        => 'vertical',
      'align'         => 'right',
      'verticalAlign' => 'top',
      'x'             => -10,
      'y'             => 100,
      'borderWidth'   => 0,
    ),
    'tooltip' => array(
      'formatter' => 'voterockit_analytic_per_day_tootltip',

    ),
  );

  $chart['series'][0]['name'] = t('Donations');
  $chart['series'][1]['name'] = t('Opens');

  foreach ($results as $result) {

    if(empty($result['forDate']['iso'])) {
      continue;
    }

    $chart['xAxis']['categories'][] = date('Y-m-d', strtotime($result['forDate']['iso']));

    if(empty($result['donations'])) {
      $result['donations'] = 0;
    }

    if(empty($result['opens'])) {
      $result['opens'] = 0;
    }

    $chart['series'][0]['data'][] = $result['donations'];
    $chart['series'][1]['data'][] = $result['opens'];
  }

  return array(
    'status'  => 'success',
    'message' => '',
    'chart'   => $chart,
    'grid'    => '',
  );
}

/**
 * voter analytics
 *
 * @param array $data
 */
function voterockit_analytic_voter($data) {

  $voteorckit = create_voterockit_object(VOTEROCKIT_BASE);
  $results = $voteorckit->query('getNeighbors');
  $results = $results['result'];


  if (empty($results)) {
    return array(
      'status'  => 'fail',
      'message' => t('Parse data not available or unreachable.'),
    );
  }

  $chart = array(
    'chart'       => array(
      'plotBackgroundColor' => NULL,
      'plotBorderWidth'     => NULL,
      'plotShadow'          => TRUE,
      'height'              => 400,
    ),
    'title'       => array(
      'text' => t('Voting Result'),
    ),
    'plotOptions' => array(
      'pie' => array(
        'allowPointSelect' => TRUE,
        'cursor'           => 'pointer',
        'dataLabels'       => array(
          'enabled'   => TRUE,
          'formatter' => 'voterockit_analytic_voter_formatter',
        ),
        'showInLegend'     => TRUE,
      ),
    ),
    'series'      => array(
      array(
        'type' => 'pie',
        'name' => t('Total'),
      )
    ),
  );

  $grid = array();

  $grid['grid']['sPaginationType'] = 'full_numbers';
  $grid['grid']['oLanguage'] = array('sSearch' => t('Search all columns:'));

  $grid['grid']['aoColumns'] = array(
    array(
      'sTitle' => t('Name'),
    ),
    array(
      'sTitle' => t('Email'),
    ),
    array(
      'sTitle' => t('Status'),
    ),
    array(
      'sTitle' => t('City'),
    ),
    array(
      'sTitle' => t('State'),
    ),
    array(
      'sTitle' => t('Zip'),
    ),
    array(
      'sTitle' => t('Address'),
    ),
  );

  $chart['series'][0]['data'] = array(
    array(
      'name'     => t('Voting'),
      'y'        => 0,
      'sliced'   => TRUE,
      'selected' => TRUE,
    ),
    array(
      t('Not Voting'),
      0
    ),
    array(
      t('Undecided'),
      0
    ),
  );

  foreach ($results as $result) {
    $grid['grid']['aaData'][] = array(
      $result['name'],
      $result['email'],
      $result['voteStatus'],
      $result['city'],
      $result['state'],
      $result['zip'],
      $result['address']
    );

    if ($result['voteStatus'] == 'Voting') {
      $chart['series'][0]['data'][0]['y']++;
    }
    elseif ($result['voteStatus'] == 'Not Voting') {
      $chart['series'][0]['data'][1][1]++;
    }
    elseif ($result['voteStatus'] == 'Undecided') {
      $chart['series'][0]['data'][2][1]++;
    }
  }

  $grid['grid_footer'] = '
  <table cellpadding="0" cellspacing="0" border="0" class="display" id="voterockit_analytic_grid">
    <tfoot>
      <tr>
        <th><input type="text" name="search_name" value="' . t('Search Name') . '" class="search_init" /></th>
        <th><input type="text" name="search_email" value="' . t('Search Email') . '" class="search_init" /></th>
         <th><input type="text" name="search_status" value="' . t('Search Status') . '" class="search_init" /></th>
        <th><input type="text" name="search_city" value="' . t('Search City') . '" class="search_init" /></th>
        <th><input type="text" name="search_state" value="' . t('Search State') . '" class="search_init" /></th>
        <th><input type="text" name="search_zip" value="' . t('Search Zip') . '" class="search_init" /></th>
        <th>&nbsp;</th>
      </tr>
    </tfoot>
</table>';

  return array(
    'status'  => 'success',
    'message' => '',
    'chart'   => $chart,
    'grid'    => $grid,
  );
}

/**
 * gear analytics
 *
 * @param array $data
 */
function voterockit_analytic_gears($data) {

  $voteorckit = create_voterockit_object(VOTEROCKIT_BASE);
  $results = $voteorckit->query('getOrderedGearItems');
  $orders = $results['result'];

  $results = $voteorckit->query('getGearItems');
  $gears = $results['result'];


  if (empty($orders) || empty($gears)) {
    return array(
      'status'  => 'fail',
      'message' => t('Parse data not available or unreachable.'),
    );
  }

  $grid = array();

  $grid['grid']['sPaginationType'] = 'full_numbers';
  $grid['grid']['oLanguage'] = array('sSearch' => t('Search all columns:'));

  $grid['grid']['aoColumns'] = array(
    array(
      'sTitle' => t('Gear'),
    ),
    array(
      'sTitle' => t('Price'),
    ),
    array(
      'sTitle' => t('Address'),
    ),
    array(
      'sTitle' => t('City'),
    ),
    array(
      'sTitle' => t('State'),
    ),
    array(
      'sTitle' => t('Zip'),
    ),
    array(
      'sTitle' => t('Image'),
    ),
  );

  foreach ($orders as $order) {

    foreach ($gears as $gear) {
      if ($order['gearItem']['objectId'] == $gear['objectId']) {
        $price = '$' . $gear['price'];
        $photo = '<button class="voterockit_analytic_gears_photo btn btn-small" content="' . $gear['photo']['url'] . '"><i class="icon-picture"></i></button>';
        $title = $gear['title'];
        break;
      }
    }

    $grid['grid']['aaData'][] = array(
      $title,
      $price,
      $order['shipAddress'],
      $order['shipCity'],
      $order['shipState'],
      $order['shipZip'],
      $photo
    );
  }

  $grid['grid_footer'] = '
  <table cellpadding="0" cellspacing="0" border="0" class="display" id="voterockit_analytic_grid">
    <tfoot>
      <tr>
        <th><input type="text" name="search_gear" value="' . t('Search Gear') . '" class="search_init" /></th>
        <th><input type="text" name="search_price" value="' . t('Search Price') . '" class="search_init" /></th>
        <th><input type="text" name="search_address" value="' . t('Search Address') . '" class="search_init" /></th>
        <th><input type="text" name="search_city" value="' . t('Search City') . '" class="search_init" /></th>
        <th><input type="text" name="search_state" value="' . t('Search State') . '" class="search_init" /></th>
        <th><input type="text" name="search_zip" value="' . t('Search Zip') . '" class="search_init" /></th>
        <th>&nbsp;</th>
      </tr>
    </tfoot>
</table>';

  return array(
    'status'  => 'success',
    'message' => '',
    'chart'   => '',
    'grid'    => $grid,
  );
}
